define HELP_TEXT
Incorrect configuration

You must define the following variables:

  TRUNK         main launchpad branch of your project
  TARBALL       name of the tarball in produces

You also need to define the following targets

  build-payload       build the tarball (code is in $$(BRANCH))
  setup-environment   run after deployer to set up environment
  deploy-build-label  apply tarball from unit's files/ dir

You can also override the following defaults

  BRANCH           checkout of your project, default: TRUNK
  BUILD_LABEL      defaults to tip
  DEPLOYER_CONFIG  deployer config, default: deploy.yaml
  DEPLOYER_TARGET  name of deployer target in config

..
endef

ifndef TRUNK
$(error $(HELP_TEXT))
endif
ifndef TARBALL
$(error $(HELP_TEXT))
endif

MAKEFLAGS += --no-builtin-rules
ENV = .venv
PYTHON = $(ENV)/bin/python
PIP = $(ENV)/bin/pip
ANSIBLE_PLAYBOOK = $(ENV)/bin/ansible-playbook
FLAKE8 = $(ENV)/bin/flake8

SHARED_ROLES ?= wsgi-app nrpe-external-master payload directories-and-permissions
UNIT_TESTS ?= unit_tests
DEPLOYER_CONFIG ?= deploy.yaml
DEPLOYER_TARGET ?= 

CHARM ?= $(shell grep name: metadata.yaml | awk '{print $$2}')
CHARM_SERIES ?= trusty

# workarond for lack of multiline vars
define NEWLINE


endef

define GET_SVC
import yaml
e = yaml.load(open('$(DEPLOYER_CONFIG)'))
for env in e.values():
    for name, svc in env['services'].items():
        if '$(CHARM)' in svc['charm']:
            print name

endef

SERVICE = $(shell echo "$(subst $(NEWLINE),@NEWLINE@,${GET_SVC})" | sed 's/@NEWLINE@/\n/g' | python -)
UNIT = $(SERVICE)/0
CHARM_UNIT_PATH ?= /var/lib/juju/agents/unit-$(SERVICE)-0/charm

# cache paths
CACHE_DIR ?= /tmp/$(CHARM)-charm-workspace
LOCAL_TRUNK = $(CACHE_DIR)/trunk
SOURCEDEPS_DIR ?= $(CACHE_DIR)/sourcedeps
JUJU_REPOSITORY ?= $(CACHE_DIR)/juju-repo
TARBALLS ?= $(CACHE_DIR)/tarballs
CUSTOM_TARBALLS ?= $(CACHE_DIR)/custom
CHARM_HELPERS_SYNC ?= $(CACHE_DIR)/charm_helpers_sync.py
ROLES_CHECKOUT ?= $(CACHE_DIR)/charm-ansible-roles
ROLES_BRANCH ?= https://github.com/absoludity/charm-ansible-roles.git

ifndef BRANCH
  ifndef BUILD_LABEL
    BUILD_LABEL = r$(shell bzr revno $(LOCAL_TRUNK))
  endif
  USING_TRUNK = true
  BRANCH = $(CACHE_DIR)/$(BUILD_LABEL)
  ASSET = $(TARBALLS)/$(BUILD_LABEL)/$(TARBALL)
else # custom BRANCH defined
  ifndef BUILD_LABEL
    BUILD_LABEL = r$(shell bzr revno $(BRANCH))
  endif
  ASSET=$(CUSTOM_TARBALLS)/$(BUILD_LABEL)/$(TARBALL)
endif

export JUJU_REPOSITORY


### development helpers ###

.ansible_hosts:
	@echo "localhost" > .ansible_hosts
.INTERMEDIATE: .ansible_hosts
.DELETE_ON_ERROR: .ansible_hosts

ansible-lint: $(ENV) .ansible_hosts
	$(ANSIBLE_PLAYBOOK) playbook.yaml --syntax-check -i .ansible_hosts

flake8: $(ENV)
	@$(FLAKE8) --exclude hooks/charmhelpers --ignore=E125 hooks
	@$(FLAKE8) --exclude hooks/charmhelpers --ignore=E125 $(UNIT_TESTS)

proof:
	@charm proof # currently useless

clean:
	rm -rf exec.d/basenode
	rm -rf files/*
	rm -f $(CHARM_HELPERS_SYNC)
	rm -rf $(ROLES_CHECKOUT)
	rm -rf $(ENV)

fullclean: clean
	rm -rf $(CACHE_DIR)


### dependency management ###

$(ENV):
	@virtualenv $(ENV)
	@$(PIP) install ansible flake8
	@[ -f requirements.txt ] && $(PIP) install -r requirements.txt || true

$(CHARM_HELPERS_SYNC):
	@bzr cat lp:charm-helpers/tools/charm_helpers_sync/charm_helpers_sync.py > $(CHARM_HELPERS_SYNC)

sync-charm-helpers: $(CHARM_HELPERS_SYNC)
	@python $(CHARM_HELPERS_SYNC) -c charm-helpers.yaml

$(ROLES_CHECKOUT):
	@git clone $(ROLES_BRANCH) $(ROLES_CHECKOUT)

sync-ansible-roles: $(ROLES_CHECKOUT)
	@mkdir -p roles
	@echo "Removing and re-creating reusable roles from sources..."
	@for role in $(SHARED_ROLES); do rm -rf roles/$$role; done
	@cd $(ROLES_CHECKOUT) && git pull
	@for role in $(SHARED_ROLES); do cp -a $(ROLES_CHECKOUT)/$$role ./roles; done


### building payloads ###

# cache dir holds all the artifacts, sourcedeps, and checkouts
$(CACHE_DIR):
	@echo Building charm workspace cache in $(CACHE_DIR)...
	@mkdir -p $(CACHE_DIR) $(TARBALLS) $(CUSTOM_TARBALLS)
	@bzr branch $(TRUNK) $(LOCAL_TRUNK)

# | means only will fire if dir doesn't exist, not if anything inside dir has changed
# http://www.gnu.org/software/make/manual/make.html#Prerequisite-Types
_pull-trunk: |$(CACHE_DIR)
	@echo Updating local copy for $(TRUNK)
	@bzr pull -d $(LOCAL_TRUNK)

# important the pull-trunk is separate target, or else the default
# BUILD_LABEL will be evaluated before pull, and thus might not be tip
_branch-trunk:
	@echo Branching $(BRANCH)
	@bzr branch $(LOCAL_TRUNK) -r $(shell echo $(BUILD_LABEL) | cut -c 2-) $(BRANCH)

# will build payload, intelligently skipping bits
ifdef USING_TRUNK
payload: _pull-trunk
	@test -d $(BRANCH) && echo $(BUILD_LABEL) code already branched || $(MAKE) _branch-trunk
	@test -f $(ASSET) && echo $(BUILD_LABEL) tarball already built || $(MAKE) build-payload BUILD_DIR=$(TARBALLS)
else  # always build when using a custom branch
payload:
	$(MAKE) build-payload BUILD_DIR=$(CUSTOM_TARBALLS)  # HOOK into your Makefile
endif


### deployment helpers ###

exec.d/basenode:
	@mkdir -p exec.d
	@bzr branch lp:~canonical-sysadmins/basenode/trunk exec.d/basenode

$(JUJU_REPOSITORY):
	@mkdir -p $(JUJU_REPOSITORY)/$(CHARM_SERIES)
	@rm -f $(JUJU_REPOSITORY)/$(CHARM_SERIES)/$(CHARM)
	@ln -sf $(PWD) $(JUJU_REPOSITORY)/$(CHARM_SERIES)/$(CHARM)

# deploy the env from scratch
deploy: payload | $(JUJU_REPOSITORY) exec.d/basenode
	juju-deployer -d --local-mods -c $(DEPLOYER_CONFIG) $(DEPLOYER_TARGET)
	$(MAKE) setup-environment  # HOOK into your Makefile
	$(MAKE) rollout SKIP_BUILD=true

# deploy a new revision/branch
rollout: _PATH=$(CHARM_UNIT_PATH)/files/$(BUILD_LABEL)
rollout:
ifneq ($(SKIP_BUILD),true)
	$(MAKE) payload
endif
	# manually copy our asset to be in the right place, rather than upgrade-charm
	juju scp $(ASSET) $(UNIT):/tmp/$(TARBALL)
	juju ssh $(UNIT) 'sudo mkdir -p $(_PATH) && sudo mv /tmp/$(TARBALL) $(_PATH)/'
	$(MAKE) deploy-build-label  # HOOK into your Makefile

upgrade-charm:
	juju upgrade-charm $(SERVICE)

# TODO replace this with an action on the nrpe charm
nagios:
	juju run --service $(SERVICE) "grep -E -oh '/(usr|srv).*' /etc/nagios/nrpe.d/check_* | grep -v etc_bzr | sudo -u nagios -s bash"
