import ast
import imp
import os
import sys
import unittest
import yaml

from mock import patch

import jinja2


DEFAULTS = dict()
PYTHON_CONFIGS = set()

with open("config.yaml") as yamlfile:
    config_data = yaml.load(yamlfile)['options']

for name, metadata in config_data.items():
    DEFAULTS[name] = metadata['default']
    if metadata.get('content_type', '') == 'python':
        PYTHON_CONFIGS.add(name)

ENV = jinja2.Environment(
    loader=jinja2.FileSystemLoader('.'),
    undefined=jinja2.StrictUndefined
)
TEMPLATE = ENV.get_template('templates/settings.py.j2')


# map between python names and config.yaml names
CONFIG_NAMES = {
    "ADMINAUDIT_EMAILS_RECIPIENTS": "adminaudit_emails_recipients",
    "CAPTCHA_PRIVATE_KEY": "captcha_private_key",
    "CAPTCHA_PUBLIC_KEY": "captcha_public_key",
    "COMBINE": "combine",
    "EMAIL_USER": "email_user",
    "EMAIL_PASSWORD": "email_password",
    "EMAIL_USE_TLS": "email_use_tls",
    "EMAIL_WHITELIST_REGEXP_LIST": "email_whitelist_regexp_list",
    "EMBEDDED_TRUST_ROOT": "embedded_trust_root",
    "GOOGLE_ANALYTICS_ID": "google_analytics_id",
    "OPENID_LAUNCHPAD_STAFF_TEAMS": "openid_launchpad_staff_teams",
    "OPENID_LAUNCHPAD_TEAMS_MAPPING": "openid_launchpad_teams_mapping",
    "SAML2IDP_REMOTES": "saml_remotes",
    "SECRET_KEY": "secret_key",
    "THROTTLE_WHITELIST_EMAIL_REGEXP_LIST": "throttle_whitelist_email_regexp_list",  # noqa
    "TWOFACTOR_FAILURE_NOTIFICATION_EMAILS": "twofactor_failure_notification_emails",  # noqa
    "TWOFACTOR_FAILURE_NOTIFICATION_TEAMS": "twofactor_failure_notification_teams",  # noqa
    "TWOFACTOR_MANDATORY_TEAMS": "twofactor_mandatory_teams",  # noqa
    "TWOFACTOR_SERVICE_IDENT": "twofactor_service_ident",
    "STATSD_PREFIX": "statsd_prefix",
}

# for settings that are not rendered in the template unless set in the config
CONDITIONAL_SETTINGS = set(['saml_remotes'])


class CharmConfigTestCase(unittest.TestCase):

    def setUp(self):
        super(CharmConfigTestCase, self).setUp()

        # mock things in settings_base module that we modify, rather than
        # overwrite
        m = imp.new_module('settings_base')
        m.OOPSES = {'publishers': [], 'template': {}}
        m.DATABASES = {'default': {}}
        m.CACHES = {'default': {}}
        m.SAML2IDP_CONFIG = {}
        m.CAPTCHA_PROXIES = {}

        # mock os module's envvars
        self.environ = {}
        p = patch.object(os, 'environ', self.environ)
        p.start()
        self.addCleanup(p.stop)

        p = patch.dict(
            sys.modules,
            {
                'django_project': m,
                'django_project.settings_base': m,
                'django_project.settings_devel': m,
            }
        )
        p.start()
        self.addCleanup(p.stop)

        self.context = self.get_template_context()

    def get_template_context(self):
        context = DEFAULTS.copy()
        context.update({
            # juju vars
            "charm_dir": "CHARM_DIR",
            "current_relation": {},
            "local_unit": "sso/0",
            "unit_private_address": "10.0.3.1",
            "relations": {
                "memcached": [],
                "nrpe-external-master": [],
                "website": [],
                "wsgi-file": [],
            },
            # playbook vars
            "hostname": "login.ubuntu.com",
            "port": 8080,
            "hostdir": "/hostdir",
            "basedir": "/basedir",
            "log_dir": "/logdir",
            "db_readonly_path": "/tmp",
            "google_saml_key_path": "/saml/key",
            "google_saml_cert_path": "/saml/cert",
        })
        return context

    def do_render(self, context):
        output = TEMPLATE.render(**context)
        settings = imp.new_module('settings')
        exec output in settings.__dict__
        return settings

    def assert_settings(self, settings, **kwargs):
        expected_config = DEFAULTS.copy()
        expected_config.update(**kwargs)

        # check that env vars were set properly
        if expected_config['deployment'] == 'devel':
            scheme = "http://"
            hostname = "10.0.3.1:8080"
            lp_hostname = hostname
        else:
            scheme = "https://"
            hostname = "login.ubuntu.com"
            lp_hostname = "login.launchpad.net"
        self.assertEqual(self.environ['SSO_ROOT_URL'], scheme + hostname)
        self.assertEqual(self.environ['SSO_HOST_DIR'], '/hostdir')
        self.assertEqual(
            self.environ['SSO_LOGS_DIR'],
            expected_config.get('SSO_LOGS_DIR', '/logdir')
        )

        # simple 1-to-1 settings tests
        for settings_name, config_name in CONFIG_NAMES.items():
            expected = expected_config[config_name]
            if config_name in CONDITIONAL_SETTINGS and not expected:
                with self.assertRaises(AttributeError):
                    getattr(settings, settings_name)
            else:
                if config_name in PYTHON_CONFIGS and expected:
                    expected = ast.literal_eval(expected)
                self.assertEqual(getattr(settings, settings_name), expected)

        # more complex specific tests
        self.assertEqual(
            settings.ALLOWED_HOSTS, [hostname, lp_hostname, "sso-app"])
        self.assertEqual(settings.API_HOST, 'http://localhost:8080')
        self.assertEqual(settings.DBFAILOVER_FLAG_DIR, '/tmp')
        self.assertEqual(settings.EMAIL_HOST, 'localhost')
        self.assertEqual(settings.EMAIL_PORT, 25)
        DB = settings.DATABASES['default']
        self.assertEqual(DB['USER'], expected_config['db_user'])
        self.assertEqual(DB['PASSWORD'], expected_config['db_password'])
        self.assertEqual(DB['NAME'], expected_config['db_name'])
        self.assertEqual(DB['HOST'], expected_config['db_host'])
        self.assertEqual(DB['PORT'], expected_config['db_port'])

        if expected_config.get('oops_config', "") != "":
            expected = {}
            expected.update(ast.literal_eval(expected_config['oops_config']))
            if expected_config.get('oops_password', "") != "":
                expected['password'] = expected_config['oops_password']
            self.assertEqual(settings.OOPSES['publishers'][0], expected)
        else:
            self.assertEqual(len(settings.OOPSES['publishers']), 0)

        if expected_config.get('statsd_hostport', "") != "":
            host, port = expected_config['statsd_hostport'].split(':')
            self.assertEqual(settings.STATSD_HOST, host)
            self.assertEqual(settings.STATSD_PORT, int(port))
        else:
            with self.assertRaises(AttributeError):
                settings.STATSD_HOST
            with self.assertRaises(AttributeError):
                settings.STATSD_PORT

        key = expected_config.get('google_saml_key', '')
        cert = expected_config.get('google_saml_cert', '')
        if key and cert:
            self.assertEqual(
                settings.SAML2IDP_CONFIG['certificate_file'], '/saml/cert')
            self.assertEqual(
                settings.SAML2IDP_CONFIG['private_key_file'], '/saml/key')

        http = expected_config.get('http_captcha_proxy')
        https = expected_config.get('https_captcha_proxy')

        if http == "" and https == "":
            self.assertEqual(settings.CAPTCHA_PROXIES, {})

        if http != "":
            self.assertEqual(
                settings.CAPTCHA_PROXIES['http'],
                expected_config.get('http_captcha_proxy'),
            )
        else:
            self.assertFalse('http' in settings.CAPTCHA_PROXIES)

        if https != "":
            self.assertEqual(
                settings.CAPTCHA_PROXIES['https'],
                expected_config.get('https_captcha_proxy'),
            )
        else:
            self.assertFalse('https' in settings.CAPTCHA_PROXIES)

    def test_defaults(self):
        settings = self.do_render(self.context)
        self.assert_settings(settings)

    def test_oops_config(self):
        oops = '{"a": "b"}'
        self.context['oops_config'] = oops
        settings = self.do_render(self.context)
        self.assert_settings(settings, oops_config=oops)

    def test_statds_hostport(self):
        hostport = "localhost:777"
        self.context['statsd_hostport'] = hostport
        settings = self.do_render(self.context)
        self.assert_settings(settings, statsd_hostport=hostport)

    def test_environment_variable_overrides(self):
        self.environ['DB_USER'] = 'superuser'
        self.environ['DB_PASSWORD'] = '1337'
        self.environ['SSO_LOGS_DIR'] = '/logs'
        settings = self.do_render(self.context)
        self.assert_settings(
            settings,
            db_user='superuser',
            db_password='1337',
            SSO_LOGS_DIR='/logs',
        )

    def test_memcached(self):
        self.context['relations']['memcached'] = [
            {
                '__relid__': '1',
                '__unit__': '1',
                'private-address': '1.2.3.4',
                'port': '1234'
            },
            {
                '__relid__': '2',
                '__unit__': '2',
                'private-address': '1.2.3.5',
            },
        ]
        settings = self.do_render(self.context)
        self.assert_settings(settings)
        self.assertEqual(
            settings.CACHES['default']['BACKEND'],
            'django.core.cache.backends.memcached.MemcachedCache'
        )
        locations = settings.CACHES['default']['LOCATION']
        self.assertEqual(len(locations), 1)
        self.assertEqual(locations[0], '1.2.3.4:1234')

    def test_memcached_no_relations(self):
        settings = self.do_render(self.context)
        self.assert_settings(settings)
        self.assertEqual(
            settings.CACHES['default']['BACKEND'],
            'django.core.cache.backends.locmem.LocMemCache'
        )
        self.assertNotIn('LOCATION', settings.CACHES['default'])

    def test_memcached_no_port(self):
        self.context['relations']['memcached'] = [
            {
                '__relid__': '1',
                '__unit__': '1',
                'private-address': '1.2.3.4',
                # no port
            },
        ]
        settings = self.do_render(self.context)
        self.assert_settings(settings)
        self.assertEqual(
            settings.CACHES['default']['BACKEND'],
            'django.core.cache.backends.locmem.LocMemCache'
        )
        self.assertNotIn('LOCATION', settings.CACHES['default'])

    def test_saml_files(self):
        self.context['google_saml_key'] = 'a'
        self.context['google_saml_cert'] = 'a'
        settings = self.do_render(self.context)
        self.assert_settings(settings)

    def test_http_proxy(self):
        self.context['http_captcha_proxy'] = 'HTTP'
        settings = self.do_render(self.context)
        self.assert_settings(
            settings,
            http_captcha_proxy='HTTP',
        )

    def test_https_proxy(self):
        self.context['https_captcha_proxy'] = 'HTTPS'
        settings = self.do_render(self.context)
        self.assert_settings(
            settings,
            https_captcha_proxy='HTTPS'
        )
